/*
 * Copyright 2019, OpenTelemetry Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.opentelemetry.opentracingshim;

import static org.junit.Assert.assertEquals;

import io.opentelemetry.OpenTelemetry;
import io.opentelemetry.sdk.correlationcontext.CorrelationContextManagerSdk;
import io.opentelemetry.sdk.trace.TracerSdkRegistry;
import org.junit.Test;

public class TraceShimTest {

  @Test
  public void createTracerShim_default() {
    TracerShim tracerShim = (TracerShim) TraceShim.createTracerShim();
    assertEquals(OpenTelemetry.getTracerRegistry().get("opentracingshim"), tracerShim.tracer());
    assertEquals(OpenTelemetry.getCorrelationContextManager(), tracerShim.contextManager());
  }

  @Test(expected = NullPointerException.class)
  public void createTracerShim_nullTracer() {
    TraceShim.createTracerShim(null, OpenTelemetry.getCorrelationContextManager());
  }

  @Test(expected = NullPointerException.class)
  public void createTracerShim_nullContextManager() {
    TraceShim.createTracerShim(OpenTelemetry.getTracerRegistry(), null);
  }

  @Test
  public void createTracerShim() {
    TracerSdkRegistry sdk = TracerSdkRegistry.builder().build();
    CorrelationContextManagerSdk contextManager = new CorrelationContextManagerSdk();
    TracerShim tracerShim = (TracerShim) TraceShim.createTracerShim(sdk, contextManager);
    assertEquals(sdk.get("opentracingshim"), tracerShim.tracer());
    assertEquals(contextManager, tracerShim.contextManager());
  }
}
